package com.windofkeltia.processor;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.nifi.annotation.behavior.DynamicProperty;
import org.apache.nifi.annotation.behavior.SideEffectFree;
import org.apache.nifi.annotation.documentation.CapabilityDescription;
import org.apache.nifi.annotation.documentation.Tags;
import org.apache.nifi.components.PropertyDescriptor;
import org.apache.nifi.components.Validator;
import org.apache.nifi.flowfile.FlowFile;
import org.apache.nifi.processor.AbstractProcessor;
import org.apache.nifi.processor.ProcessContext;
import org.apache.nifi.processor.ProcessSession;
import org.apache.nifi.processor.ProcessorInitializationContext;
import org.apache.nifi.processor.Relationship;
import org.apache.nifi.processor.exception.ProcessException;
import org.apache.nifi.processor.io.StreamCallback;

import com.windofkeltia.utilities.StreamUtilities;

@SideEffectFree
@Tags( { "sample" } )
@CapabilityDescription( "Illustrates a simple search-and-replace processor." )
@DynamicProperty( name        = "(search string)",
                  value       = "(replacement string)",
                  description = "Search in-coming text for the search string(s) and substitute using the indicated "
                              + " and corresponding replacement string(s)." )
public class Filter extends AbstractProcessor
{
  @Override
  public void onTrigger( ProcessContext context, ProcessSession session ) throws ProcessException
  {
    FlowFile flowfile = session.get();

    if( flowfile == null )
    {
      context.yield();
      return;
    }

    final Map< String, String > replacements = new HashMap<>();   // [ find, replace ]

    // get our search- and replace strings from dynamic properties configured by our user...
    for( Map.Entry< PropertyDescriptor, String > descriptor : context.getProperties().entrySet() )
    {
      PropertyDescriptor property = descriptor.getKey();
      if( !property.isDynamic() )
        continue;

      replacements.put( property.getName(), descriptor.getValue() );
    }

    if( replacements.size() == 0 )
    {
      session.transfer( flowfile, FAILURE );
      return;
    }

    FlowFile rewrite = session.write( flowfile, new StreamCallback()
    {
      @Override
      public void process( InputStream inputStream, OutputStream outputStream ) throws IOException
      {
        // read the in-coming flowfile, ...
        String input = StreamUtilities.slurp( inputStream, 8192 );

        // ...search for all the strings and replace them, ...
        for( Map.Entry< String, String > replace : replacements.entrySet() )
          input = input.replaceAll( replace.getKey(), replace.getValue() );

        // ...then write out the resulting output.
        outputStream.write( input.getBytes() );
      }
    } );

    session.transfer( rewrite, SUCCESS );
  }

  private final Set< String > dynamicPropertyNames = new HashSet<>();

  @Override
  protected PropertyDescriptor getSupportedDynamicPropertyDescriptor( final String propertyDescriptorName )
  {
    return new PropertyDescriptor.Builder()
        .required( false )
        .name( propertyDescriptorName )
        .addValidator( Validator.VALID )
        .dynamic( true )
        .build();
  }

  //<editor-fold desc="Properties (0) and relationships...">
  public static final Relationship SUCCESS = new Relationship.Builder()
      .name( "success" )
      .description( "The modified flowfile..." )
      .build();
  public static final Relationship FAILURE = new Relationship.Builder()
      .name( "failure" )
      .description( "The failing flowfile..." )
      .build();

  private List< PropertyDescriptor > properties;
  private Set< Relationship >        relationships;

  @Override
  public void init( final ProcessorInitializationContext context )
  {
    //noinspection MismatchedQueryAndUpdateOfCollection
    List< PropertyDescriptor > properties = new ArrayList<>();
    this.properties = Collections.unmodifiableList( properties );

    Set< Relationship > relationships = new HashSet<>();
    relationships.add( SUCCESS );
    relationships.add( FAILURE );
    this.relationships = Collections.unmodifiableSet( relationships );
  }

  @Override public List< PropertyDescriptor > getSupportedPropertyDescriptors() { return properties; }
  @Override public Set< Relationship >        getRelationships()                { return relationships; }
  //</editor-fold>
}