/*
 * $Id$
 *
 * Proprietary and confidential.
 * Copyright $Date::      $ Perfect Search Corporation.
 * All rights reserved.
 */
package com.windofkeltia.utilities;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.nio.charset.StandardCharsets;


import static java.util.Objects.isNull;

/**
 * @author Russell Bateman
 * @since 2015
 */
public class StreamUtilities
{
   /**
   * "Slurp" all the bytes on the stream using a buffer size of 4096 into a
   * StringBuilder and return it. The <tt>InputStream</tt> is not closed
   * after use as it's managed by the caller.
   * @param is the input stream.
   * @return the bytes in the stream as a StringBuilder.
   * @throws IOException from I/O operations.
   */
  public static StringBuilder slurpIntoBuilder( final InputStream is ) throws IOException
  {
    return slurpIntoBuilder( is, 4096 );
  }

  /**
   * "Slurp" all the bytes on the stream into a String and return it.
   * See {@link #slurpIntoBuilder( InputStream, int )}.
   */
  public static String slurp( final InputStream is, final int bufferSize ) throws IOException
  {
    return slurpIntoBuilder( is, bufferSize ).toString();
  }

  /**
   * "Slurp" all the bytes on the stream into a StringBuilder and return it. The
   * <tt>InputStream</tt> is not closed after use as it's managed by the
   * caller.
   * @param is the input stream.
   * @param bufferSize pass -1 if you don't care how much is read at a time.
   * @return the bytes in the stream as a StringBuilder.
   * @throws IOException from I/O operations.
   */
  public static StringBuilder slurpIntoBuilder( final InputStream is, final int bufferSize ) throws IOException
  {
    int                 howMuch = ( bufferSize == -1 ) ? 4096 : bufferSize;
    final char[]        buffer  = new char[ howMuch ];
    final StringBuilder out     = new StringBuilder();
    final Reader        in      = new InputStreamReader( is, StandardCharsets.UTF_8 );

    for( ; ; )
    {
      int chunkLength = in.read( buffer, 0, buffer.length );

      if( chunkLength < 0 )
        break;

      out.append( buffer, 0, chunkLength );
    }

    return out;
  }

  /** Slurp content from input stream and write it to the output stream. */
  public static void slurpIntoOutputStream( final InputStream inputStream, final OutputStream outputStream, int bufferSize )
      throws IOException
  {
    if( isNull( inputStream ) )
      throw new IOException( "Missing input stream" );
    else if( isNull( outputStream ) )
      throw new IOException( "Missing output stream" );

    byte[] buffer = new byte[ bufferSize ];

    for( ; ; )
    {
      int length = inputStream.read( buffer, 0, bufferSize );
      outputStream.write( buffer, 0, length );

      if( length < bufferSize )
        break;
    }
  }
}
